==> QFirehose_Linux_Android_V1.4.8.zip: QFirehose_Linux_Android_V1.4.8/md5.c <==
/******************************************************************************
  @file    mdm5.c
  @brief   mdm4 checksum.

  DESCRIPTION
  QFirehoe Tool for USB and PCIE of Quectel wireless cellular modules.

  INITIALIZATION AND SEQUENCING REQUIREMENTS
  None.

  ---------------------------------------------------------------------------
  Copyright (c) 2016 - 2020 Quectel Wireless Solution, Co., Ltd.  All Rights Reserved.
  Quectel Wireless Solution Proprietary and Confidential.
  ---------------------------------------------------------------------------
******************************************************************************/
#include "md5.h"
#include <endian.h> //for __BYTE_ORDER

#define F(x, y, z)			((z) ^ ((x) & ((y) ^ (z))))
#define G(x, y, z)			((y) ^ ((z) & ((x) ^ (y))))
#define H(x, y, z)			(((x) ^ (y)) ^ (z))
#define H2(x, y, z)			((x) ^ ((y) ^ (z)))
#define I(x, y, z)			((y) ^ ((x) | ~(z)))

#define STEP(f, a, b, c, d, x, t, s) \
	(a) += f((b), (c), (d)) + (x) + (t); \
	(a) = (((a) << (s)) | (((a) & 0xffffffff) >> (32 - (s)))); \
	(a) += (b);

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define SET(n) \
	(*(uint32_t *)&ptr[(n) * 4])
#define GET(n) \
	SET(n)
#else
#define SET(n) \
	(block[(n)] = \
	(uint32_t)ptr[(n) * 4] | \
	((uint32_t)ptr[(n) * 4 + 1] << 8) | \
	((uint32_t)ptr[(n) * 4 + 2] << 16) | \
	((uint32_t)ptr[(n) * 4 + 3] << 24))
#define GET(n) \
	(block[(n)])
#endif


static const void *body(md5_ctx_t *ctx, const void *data, unsigned long size) {
    const unsigned char *ptr;
    uint32_t a, b, c, d;
    uint32_t saved_a, saved_b, saved_c, saved_d;
#if __BYTE_ORDER != __LITTLE_ENDIAN
    uint32_t block[16];
#endif

    ptr = (const unsigned char *)data;

    a = ctx->a;
    b = ctx->b;
    c = ctx->c;
    d = ctx->d;

    do {
        saved_a = a;
        saved_b = b;
        saved_c = c;
        saved_d = d;

        /* Round 1 */
        STEP(F, a, b, c, d, SET(0), 0xd76aa478, 7)
        STEP(F, d, a, b, c, SET(1), 0xe8c7b756, 12)
        STEP(F, c, d, a, b, SET(2), 0x242070db, 17)
        STEP(F, b, c, d, a, SET(3), 0xc1bdceee, 22)
        STEP(F, a, b, c, d, SET(4), 0xf57c0faf, 7)
        STEP(F, d, a, b, c, SET(5), 0x4787c62a, 12)
        STEP(F, c, d, a, b, SET(6), 0xa8304613, 17)
        STEP(F, b, c, d, a, SET(7), 0xfd469501, 22)
        STEP(F, a, b, c, d, SET(8), 0x698098d8, 7)
        STEP(F, d, a, b, c, SET(9), 0x8b44f7af, 12)
        STEP(F, c, d, a, b, SET(10), 0xffff5bb1, 17)
        STEP(F, b, c, d, a, SET(11), 0x895cd7be, 22)
        STEP(F, a, b, c, d, SET(12), 0x6b901122, 7)
        STEP(F, d, a, b, c, SET(13), 0xfd987193, 12)
        STEP(F, c, d, a, b, SET(14), 0xa679438e, 17)
        STEP(F, b, c, d, a, SET(15), 0x49b40821, 22)

        /* Round 2 */
        STEP(G, a, b, c, d, GET(1), 0xf61e2562, 5)
        STEP(G, d, a, b, c, GET(6), 0xc040b340, 9)
        STEP(G, c, d, a, b, GET(11), 0x265e5a51, 14)
        STEP(G, b, c, d, a, GET(0), 0xe9b6c7aa, 20)
        STEP(G, a, b, c, d, GET(5), 0xd62f105d, 5)
        STEP(G, d, a, b, c, GET(10), 0x02441453, 9)
        STEP(G, c, d, a, b, GET(15), 0xd8a1e681, 14)
        STEP(G, b, c, d, a, GET(4), 0xe7d3fbc8, 20)
        STEP(G, a, b, c, d, GET(9), 0x21e1cde6, 5)
        STEP(G, d, a, b, c, GET(14), 0xc33707d6, 9)
        STEP(G, c, d, a, b, GET(3), 0xf4d50d87, 14)
        STEP(G, b, c, d, a, GET(8), 0x455a14ed, 20)
        STEP(G, a, b, c, d, GET(13), 0xa9e3e905, 5)
        STEP(G, d, a, b, c, GET(2), 0xfcefa3f8, 9)
        STEP(G, c, d, a, b, GET(7), 0x676f02d9, 14)
        STEP(G, b, c, d, a, GET(12), 0x8d2a4c8a, 20)

        /* Round 3 */
        STEP(H, a, b, c, d, GET(5), 0xfffa3942, 4)
        STEP(H2, d, a, b, c, GET(8), 0x8771f681, 11)
        STEP(H, c, d, a, b, GET(11), 0x6d9d6122, 16)
        STEP(H2, b, c, d, a, GET(14), 0xfde5380c, 23)
        STEP(H, a, b, c, d, GET(1), 0xa4beea44, 4)
        STEP(H2, d, a, b, c, GET(4), 0x4bdecfa9, 11)
        STEP(H, c, d, a, b, GET(7), 0xf6bb4b60, 16)
        STEP(H2, b, c, d, a, GET(10), 0xbebfbc70, 23)
        STEP(H, a, b, c, d, GET(13), 0x289b7ec6, 4)
        STEP(H2, d, a, b, c, GET(0), 0xeaa127fa, 11)
        STEP(H, c, d, a, b, GET(3), 0xd4ef3085, 16)
        STEP(H2, b, c, d, a, GET(6), 0x04881d05, 23)
        STEP(H, a, b, c, d, GET(9), 0xd9d4d039, 4)
        STEP(H2, d, a, b, c, GET(12), 0xe6db99e5, 11)
        STEP(H, c, d, a, b, GET(15), 0x1fa27cf8, 16)
        STEP(H2, b, c, d, a, GET(2), 0xc4ac5665, 23)

        /* Round 4 */
        STEP(I, a, b, c, d, GET(0), 0xf4292244, 6)
        STEP(I, d, a, b, c, GET(7), 0x432aff97, 10)
        STEP(I, c, d, a, b, GET(14), 0xab9423a7, 15)
        STEP(I, b, c, d, a, GET(5), 0xfc93a039, 21)
        STEP(I, a, b, c, d, GET(12), 0x655b59c3, 6)
        STEP(I, d, a, b, c, GET(3), 0x8f0ccc92, 10)
        STEP(I, c, d, a, b, GET(10), 0xffeff47d, 15)
        STEP(I, b, c, d, a, GET(1), 0x85845dd1, 21)
        STEP(I, a, b, c, d, GET(8), 0x6fa87e4f, 6)
        STEP(I, d, a, b, c, GET(15), 0xfe2ce6e0, 10)
        STEP(I, c, d, a, b, GET(6), 0xa3014314, 15)
        STEP(I, b, c, d, a, GET(13), 0x4e0811a1, 21)
        STEP(I, a, b, c, d, GET(4), 0xf7537e82, 6)
        STEP(I, d, a, b, c, GET(11), 0xbd3af235, 10)
        STEP(I, c, d, a, b, GET(2), 0x2ad7d2bb, 15)
        STEP(I, b, c, d, a, GET(9), 0xeb86d391, 21)

        a += saved_a;
        b += saved_b;
        c += saved_c;
        d += saved_d;

        ptr += 64;
    } while (size -= 64);

    ctx->a = a;
    ctx->b = b;
    ctx->c = c;
    ctx->d = d;

    return ptr;
}

void md5_begin(md5_ctx_t *ctx) {
    ctx->a = 0x67452301;
    ctx->b = 0xefcdab89;
    ctx->c = 0x98badcfe;
    ctx->d = 0x10325476;

    ctx->lo = 0;
    ctx->hi = 0;
}

void md5_hash(const void *data, size_t size, md5_ctx_t *ctx) {
    uint32_t saved_lo;
    unsigned long used, available;

    saved_lo = ctx->lo;
    if ((ctx->lo = (saved_lo + size) & 0x1fffffff) < saved_lo)
        ctx->hi++;
    ctx->hi += size >> 29;

    used = saved_lo & 0x3f;

    if (used) {
        available = 64 - used;

        if (size < available) {
            memcpy(&ctx->buffer[used], data, size);
            return;
        }

        memcpy(&ctx->buffer[used], data, available);
        data = (const unsigned char *)data + available;
        size -= available;
        body(ctx, ctx->buffer, 64);
    }

    if (size >= 64) {
        data = body(ctx, data, size & ~((size_t) 0x3f));
        size &= 0x3f;
    }

    memcpy(ctx->buffer, data, size);
}

void md5_end(void *resbuf, md5_ctx_t *ctx) {
    unsigned char *result = (unsigned char*)resbuf;
    unsigned long used, available;

    used = ctx->lo & 0x3f;

    ctx->buffer[used++] = 0x80;

    available = 64 - used;

    if (available < 8) {
        memset(&ctx->buffer[used], 0, available);
        body(ctx, ctx->buffer, 64);
        used = 0;
        available = 64;
    }

    memset(&ctx->buffer[used], 0, available - 8);

    ctx->lo <<= 3;
    ctx->buffer[56] = ctx->lo;
    ctx->buffer[57] = ctx->lo >> 8;
    ctx->buffer[58] = ctx->lo >> 16;
    ctx->buffer[59] = ctx->lo >> 24;
    ctx->buffer[60] = ctx->hi;
    ctx->buffer[61] = ctx->hi >> 8;
    ctx->buffer[62] = ctx->hi >> 16;
    ctx->buffer[63] = ctx->hi >> 24;

    body(ctx, ctx->buffer, 64);

    result[0] = ctx->a;
    result[1] = ctx->a >> 8;
    result[2] = ctx->a >> 16;
    result[3] = ctx->a >> 24;
    result[4] = ctx->b;
    result[5] = ctx->b >> 8;
    result[6] = ctx->b >> 16;
    result[7] = ctx->b >> 24;
    result[8] = ctx->c;
    result[9] = ctx->c >> 8;
    result[10] = ctx->c >> 16;
    result[11] = ctx->c >> 24;
    result[12] = ctx->d;
    result[13] = ctx->d >> 8;
    result[14] = ctx->d >> 16;
    result[15] = ctx->d >> 24;

    memset(ctx, 0, sizeof(*ctx));
}

int md5sum(char *file, void *md5_buf) {
    char buf[256];
    md5_ctx_t ctx;
    int ret = 0;
    FILE *f;

    f = fopen(file, "r");
    if (!f)
        return -1;

    md5_begin(&ctx);
    do {
        int len = fread(buf, 1, sizeof(buf), f);
        if (!len)
            break;

        md5_hash(buf, len, &ctx);
        ret += len;
    } while(1);

    md5_end(md5_buf, &ctx);
    fclose(f);

    return ret;
}

int md5_check(const char *firehose_dir)
{
	FILE *fp = NULL;
	char md5_file_path[256], buff[256], file_name[128], file_full_path[256], file_md5_value[64];
	char *ps = NULL, *pe = NULL;
	unsigned char compute_md5_buff[16];
	char convert_md5_buff[33];
	int i, file_count = 0, fail_count = 0;

	snprintf(md5_file_path, sizeof(md5_file_path), "%s/md5.txt", firehose_dir);
	if (access(md5_file_path, R_OK)) {
		dbg_time("Cann't find md5.txt in %s, Please check it!\n", firehose_dir);
		return -1;
	} else {
		dbg_time("Find md5 check file <%s>\n", md5_file_path);
	}
	
	fp = fopen(md5_file_path, "rb");
	if (fp == NULL) {
		dbg_time("fail to fopen(%s), error: %d (%s)\n", md5_file_path, errno, strerror(errno));
		return -1;
	}

	while (fgets(buff, sizeof(buff), fp)) {
		if (strstr(buff, "targetfiles.zip"))
			continue;
		
		ps = strstr(buff, ":\\");
		if (ps == NULL)
			continue;

		file_count++;
		ps = ps + 1;
		pe = strstr(ps, ":");
		memcpy(file_name, ps, pe - ps);
		file_name[pe-ps] = '\0';

		for (i = 0; file_name[i] != '\0'; i++) {
			if (file_name[i] == '\\')
				file_name[i] = '/';
		}

		memcpy(file_md5_value, pe + 1, 32);
		file_md5_value[32] = '\0';
		snprintf(file_full_path, sizeof(file_full_path), "%s%s", firehose_dir, file_name);
		
		md5sum(file_full_path, compute_md5_buff);
		
		for (i = 0; i< 16; i++) {
			sprintf(convert_md5_buff+(i*2), "%02X", compute_md5_buff[i]);	
		}
		
		if (strncasecmp(file_md5_value, convert_md5_buff, 16)) {
			fail_count++;
			dbg_time("md5 checking: %s fail\n", file_full_path);
			dbg_time("find %s, should be %s\n", file_md5_value, convert_md5_buff);
		} else {
			dbg_time("md5 checking: %s pass\n", file_full_path);
		}
	}

	fclose(fp);
	dbg_time("Totals checking %d files md5 value, %d file fail!\n", file_count, fail_count);
	
	return (fail_count? -1 : 0);
}

==> QFirehose_Linux_Android_V1.4.17.zip: QFirehose_Linux_Android_V1.4.17/md5.c <==
/*
    Copyright 2023 Quectel Wireless Solutions Co.,Ltd

    Quectel hereby grants customers of Quectel a license to use, modify,
    distribute and publish the Software in binary form provided that
    customers shall have no right to reverse engineer, reverse assemble,
    decompile or reduce to source code form any portion of the Software.
    Under no circumstances may customers modify, demonstrate, use, deliver
    or disclose any portion of the Software in source code form.
*/

#include "md5.h"
#include <endian.h> //for __BYTE_ORDER

#define F(x, y, z)			((z) ^ ((x) & ((y) ^ (z))))
#define G(x, y, z)			((y) ^ ((z) & ((x) ^ (y))))
#define H(x, y, z)			(((x) ^ (y)) ^ (z))
#define H2(x, y, z)			((x) ^ ((y) ^ (z)))
#define I(x, y, z)			((y) ^ ((x) | ~(z)))

#define STEP(f, a, b, c, d, x, t, s) \
	(a) += f((b), (c), (d)) + (x) + (t); \
	(a) = (((a) << (s)) | (((a) & 0xffffffff) >> (32 - (s)))); \
	(a) += (b);

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define SET(n) \
	(*(uint32_t *)&ptr[(n) * 4])
#define GET(n) \
	SET(n)
#else
#define SET(n) \
	(block[(n)] = \
	(uint32_t)ptr[(n) * 4] | \
	((uint32_t)ptr[(n) * 4 + 1] << 8) | \
	((uint32_t)ptr[(n) * 4 + 2] << 16) | \
	((uint32_t)ptr[(n) * 4 + 3] << 24))
#define GET(n) \
	(block[(n)])
#endif

extern char firehose_unzip_full_dir[256];
extern char firehose_zip_name[80];

static const void *body(md5_ctx_t *ctx, const void *data, unsigned long size) {
    const unsigned char *ptr;
    uint32_t a, b, c, d;
    uint32_t saved_a, saved_b, saved_c, saved_d;
#if __BYTE_ORDER != __LITTLE_ENDIAN
    uint32_t block[16];
#endif

    ptr = (const unsigned char *)data;

    a = ctx->a;
    b = ctx->b;
    c = ctx->c;
    d = ctx->d;

    do {
        saved_a = a;
        saved_b = b;
        saved_c = c;
        saved_d = d;

        /* Round 1 */
        STEP(F, a, b, c, d, SET(0), 0xd76aa478, 7)
        STEP(F, d, a, b, c, SET(1), 0xe8c7b756, 12)
        STEP(F, c, d, a, b, SET(2), 0x242070db, 17)
        STEP(F, b, c, d, a, SET(3), 0xc1bdceee, 22)
        STEP(F, a, b, c, d, SET(4), 0xf57c0faf, 7)
        STEP(F, d, a, b, c, SET(5), 0x4787c62a, 12)
        STEP(F, c, d, a, b, SET(6), 0xa8304613, 17)
        STEP(F, b, c, d, a, SET(7), 0xfd469501, 22)
        STEP(F, a, b, c, d, SET(8), 0x698098d8, 7)
        STEP(F, d, a, b, c, SET(9), 0x8b44f7af, 12)
        STEP(F, c, d, a, b, SET(10), 0xffff5bb1, 17)
        STEP(F, b, c, d, a, SET(11), 0x895cd7be, 22)
        STEP(F, a, b, c, d, SET(12), 0x6b901122, 7)
        STEP(F, d, a, b, c, SET(13), 0xfd987193, 12)
        STEP(F, c, d, a, b, SET(14), 0xa679438e, 17)
        STEP(F, b, c, d, a, SET(15), 0x49b40821, 22)

        /* Round 2 */
        STEP(G, a, b, c, d, GET(1), 0xf61e2562, 5)
        STEP(G, d, a, b, c, GET(6), 0xc040b340, 9)
        STEP(G, c, d, a, b, GET(11), 0x265e5a51, 14)
        STEP(G, b, c, d, a, GET(0), 0xe9b6c7aa, 20)
        STEP(G, a, b, c, d, GET(5), 0xd62f105d, 5)
        STEP(G, d, a, b, c, GET(10), 0x02441453, 9)
        STEP(G, c, d, a, b, GET(15), 0xd8a1e681, 14)
        STEP(G, b, c, d, a, GET(4), 0xe7d3fbc8, 20)
        STEP(G, a, b, c, d, GET(9), 0x21e1cde6, 5)
        STEP(G, d, a, b, c, GET(14), 0xc33707d6, 9)
        STEP(G, c, d, a, b, GET(3), 0xf4d50d87, 14)
        STEP(G, b, c, d, a, GET(8), 0x455a14ed, 20)
        STEP(G, a, b, c, d, GET(13), 0xa9e3e905, 5)
        STEP(G, d, a, b, c, GET(2), 0xfcefa3f8, 9)
        STEP(G, c, d, a, b, GET(7), 0x676f02d9, 14)
        STEP(G, b, c, d, a, GET(12), 0x8d2a4c8a, 20)

        /* Round 3 */
        STEP(H, a, b, c, d, GET(5), 0xfffa3942, 4)
        STEP(H2, d, a, b, c, GET(8), 0x8771f681, 11)
        STEP(H, c, d, a, b, GET(11), 0x6d9d6122, 16)
        STEP(H2, b, c, d, a, GET(14), 0xfde5380c, 23)
        STEP(H, a, b, c, d, GET(1), 0xa4beea44, 4)
        STEP(H2, d, a, b, c, GET(4), 0x4bdecfa9, 11)
        STEP(H, c, d, a, b, GET(7), 0xf6bb4b60, 16)
        STEP(H2, b, c, d, a, GET(10), 0xbebfbc70, 23)
        STEP(H, a, b, c, d, GET(13), 0x289b7ec6, 4)
        STEP(H2, d, a, b, c, GET(0), 0xeaa127fa, 11)
        STEP(H, c, d, a, b, GET(3), 0xd4ef3085, 16)
        STEP(H2, b, c, d, a, GET(6), 0x04881d05, 23)
        STEP(H, a, b, c, d, GET(9), 0xd9d4d039, 4)
        STEP(H2, d, a, b, c, GET(12), 0xe6db99e5, 11)
        STEP(H, c, d, a, b, GET(15), 0x1fa27cf8, 16)
        STEP(H2, b, c, d, a, GET(2), 0xc4ac5665, 23)

        /* Round 4 */
        STEP(I, a, b, c, d, GET(0), 0xf4292244, 6)
        STEP(I, d, a, b, c, GET(7), 0x432aff97, 10)
        STEP(I, c, d, a, b, GET(14), 0xab9423a7, 15)
        STEP(I, b, c, d, a, GET(5), 0xfc93a039, 21)
        STEP(I, a, b, c, d, GET(12), 0x655b59c3, 6)
        STEP(I, d, a, b, c, GET(3), 0x8f0ccc92, 10)
        STEP(I, c, d, a, b, GET(10), 0xffeff47d, 15)
        STEP(I, b, c, d, a, GET(1), 0x85845dd1, 21)
        STEP(I, a, b, c, d, GET(8), 0x6fa87e4f, 6)
        STEP(I, d, a, b, c, GET(15), 0xfe2ce6e0, 10)
        STEP(I, c, d, a, b, GET(6), 0xa3014314, 15)
        STEP(I, b, c, d, a, GET(13), 0x4e0811a1, 21)
        STEP(I, a, b, c, d, GET(4), 0xf7537e82, 6)
        STEP(I, d, a, b, c, GET(11), 0xbd3af235, 10)
        STEP(I, c, d, a, b, GET(2), 0x2ad7d2bb, 15)
        STEP(I, b, c, d, a, GET(9), 0xeb86d391, 21)

        a += saved_a;
        b += saved_b;
        c += saved_c;
        d += saved_d;

        ptr += 64;
    } while (size -= 64);

    ctx->a = a;
    ctx->b = b;
    ctx->c = c;
    ctx->d = d;

    return ptr;
}

void md5_begin(md5_ctx_t *ctx) {
    ctx->a = 0x67452301;
    ctx->b = 0xefcdab89;
    ctx->c = 0x98badcfe;
    ctx->d = 0x10325476;

    ctx->lo = 0;
    ctx->hi = 0;
}

void md5_hash(const void *data, size_t size, md5_ctx_t *ctx) {
    uint32_t saved_lo;
    unsigned long used, available;

    saved_lo = ctx->lo;
    if ((ctx->lo = (saved_lo + size) & 0x1fffffff) < saved_lo)
        ctx->hi++;
    ctx->hi += size >> 29;

    used = saved_lo & 0x3f;

    if (used) {
        available = 64 - used;

        if (size < available) {
            memcpy(&ctx->buffer[used], data, size);
            return;
        }

        memcpy(&ctx->buffer[used], data, available);
        data = (const unsigned char *)data + available;
        size -= available;
        body(ctx, ctx->buffer, 64);
    }

    if (size >= 64) {
        data = body(ctx, data, size & ~((size_t) 0x3f));
        size &= 0x3f;
    }

    memcpy(ctx->buffer, data, size);
}

void md5_end(void *resbuf, md5_ctx_t *ctx) {
    unsigned char *result = (unsigned char*)resbuf;
    unsigned long used, available;

    used = ctx->lo & 0x3f;

    ctx->buffer[used++] = 0x80;

    available = 64 - used;

    if (available < 8) {
        memset(&ctx->buffer[used], 0, available);
        body(ctx, ctx->buffer, 64);
        used = 0;
        available = 64;
    }

    if (used >= 64)
        return (void)0;

    memset(&ctx->buffer[used], 0, available - 8);

    ctx->lo <<= 3;
    ctx->buffer[56] = ctx->lo;
    ctx->buffer[57] = ctx->lo >> 8;
    ctx->buffer[58] = ctx->lo >> 16;
    ctx->buffer[59] = ctx->lo >> 24;
    ctx->buffer[60] = ctx->hi;
    ctx->buffer[61] = ctx->hi >> 8;
    ctx->buffer[62] = ctx->hi >> 16;
    ctx->buffer[63] = ctx->hi >> 24;

    body(ctx, ctx->buffer, 64);

    result[0] = ctx->a;
    result[1] = ctx->a >> 8;
    result[2] = ctx->a >> 16;
    result[3] = ctx->a >> 24;
    result[4] = ctx->b;
    result[5] = ctx->b >> 8;
    result[6] = ctx->b >> 16;
    result[7] = ctx->b >> 24;
    result[8] = ctx->c;
    result[9] = ctx->c >> 8;
    result[10] = ctx->c >> 16;
    result[11] = ctx->c >> 24;
    result[12] = ctx->d;
    result[13] = ctx->d >> 8;
    result[14] = ctx->d >> 16;
    result[15] = ctx->d >> 24;

    memset(ctx, 0, sizeof(*ctx));
}

int md5sum(char *file, void *md5_buf) {
    char buf[256];
    md5_ctx_t ctx;
    int ret = 0;
    FILE *f;

    f = fopen(file, "r");
    if (!f)
        return -1;

    md5_begin(&ctx);
    do {
        int len = fread(buf, 1, sizeof(buf), f);
        if (!len)
            break;

        md5_hash(buf, len, &ctx);
        ret += len;
    } while(1);

    md5_end(md5_buf, &ctx);
    fclose(f);

    return ret;
}

int md5_check(const char *firehose_dir)
{
	FILE *fp = NULL;
	char md5_file_path[256], buff[256], file_name[128], file_name_tmp[128], file_full_path[256], file_md5_value[64];
	char *ps = NULL, *pe = NULL;
	unsigned char compute_md5_buff[16];
	char convert_md5_buff[33];
	int i, file_count = 0, fail_count = 0;

    if (is_upgrade_fimeware_zip_7z)
    {
        memset(zip_cmd_buf, 0, sizeof(zip_cmd_buf));

        if (is_upgrade_fimeware_only_zip)
        {
            if (is_firehose_zip_7z_name_exit)
            {
                snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "unzip -o -q %.240s %.76s/'*md5.txt' -d /tmp/ > %s", firehose_dir, firehose_zip_name, ZIP_PROCESS_INFO);
            }
            else
            {
                snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "unzip -o -q %.240s '*md5.txt' -d /tmp/ > %s", firehose_dir, ZIP_PROCESS_INFO);
            }
        }
        else
        {
            if (is_firehose_zip_7z_name_exit)
            {
                snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "7z x %.240s -o/tmp/ %.76s/md5.txt > %s", firehose_dir, firehose_zip_name, ZIP_PROCESS_INFO);
            }
            else
            {
                snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "7z x %.240s -o/tmp/ md5.txt > %s", firehose_dir, ZIP_PROCESS_INFO);
            }
        }
        printf("%s zip_cmd_buf:%s\n", __func__, zip_cmd_buf);

        if( -1 == system(zip_cmd_buf))
        {
            printf("%s system return error\n", __func__);
            return -1;
        }
        usleep(1000);
        snprintf(md5_file_path, sizeof(md5_file_path), "%.240s/md5.txt", firehose_unzip_full_dir);
        if (access(md5_file_path, R_OK)) {
            dbg_time("Cann't find md5.txt in %s, Please check it!\n", md5_file_path);
            return -1;
        } else {
            dbg_time("Find md5 check file <%s>\n", md5_file_path);
        }
    }
    else
    {
        snprintf(md5_file_path, sizeof(md5_file_path), "%.240s/md5.txt", firehose_dir);
        if (access(md5_file_path, R_OK)) {
            dbg_time("Cann't find md5.txt in %s, Please check it!\n", firehose_dir);
            return 0; //allow skip md5 check by delete md5.txt
        } else {
            dbg_time("Find md5 check file <%s>\n", md5_file_path);
        }
    }

	fp = fopen(md5_file_path, "rb");
	if (fp == NULL) {
		dbg_time("fail to fopen(%s), error: %d (%s)\n", md5_file_path, errno, strerror(errno));
		return -1;
	}

	while (fgets(buff, sizeof(buff), fp)) {
		if (strstr(buff, "targetfiles.zip"))
			continue;

		ps = strstr(buff, ":\\");
		if (ps == NULL)
			continue;

		file_count++;
		ps = ps + 1;
		pe = strstr(ps, ":");
        if (pe == NULL)
			continue;

		memcpy(file_name, ps, pe - ps);
		file_name[pe-ps] = '\0';

		for (i = 0; file_name[i] != '\0'; i++) {
			if (file_name[i] == '\\')
				file_name[i] = '/';
		}

		memcpy(file_md5_value, pe + 1, 32);
		file_md5_value[32] = '\0';

        if (is_upgrade_fimeware_zip_7z)
        {
            char *p1 = strchr(file_name, '/');
            memset(file_name_tmp, 0, sizeof(file_name_tmp));
            strncpy(file_name_tmp, p1 + 1, strlen(p1) - 1);
            memset(zip_cmd_buf, 0, sizeof(zip_cmd_buf));

            if (is_upgrade_fimeware_only_zip)
            {
                if (is_firehose_zip_7z_name_exit)
                {
                    snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "unzip -o -q %.240s %.76s/'*%.120s' -d /tmp/ > %s", firehose_dir, firehose_zip_name, file_name_tmp, ZIP_PROCESS_INFO);
                }
                else
                {
                    snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "unzip -o -q %.240s '*%.120s' -d /tmp/ > %s", firehose_dir, file_name_tmp, ZIP_PROCESS_INFO);
                }
            }
            else
            {
                if (is_firehose_zip_7z_name_exit)
                {
                    snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "7z x %.240s -o/tmp/ %.76s/%.120s > %s", firehose_dir, firehose_zip_name, file_name_tmp, ZIP_PROCESS_INFO);
                }
                else
                {
                    snprintf(zip_cmd_buf, sizeof(zip_cmd_buf), "7z x %.240s -o/tmp/ %.120s > %s", firehose_dir, file_name_tmp, ZIP_PROCESS_INFO);
                }
            }
            dbg_time("%s zip_cmd_buf:%s\n", __func__, zip_cmd_buf);

            if( -1 == system(zip_cmd_buf))
            {
                dbg_time("%s system return error\n", __func__);
                return -1;
            }
            usleep(1000);

            snprintf(file_full_path, sizeof(file_full_path), "%.160s%.80s", firehose_unzip_full_dir, file_name);
            dbg_time("%s file_full_path:%s\n", __func__, file_full_path);
        }
        else
        {
            snprintf(file_full_path, sizeof(file_full_path), "%.160s%.80s", firehose_dir, file_name);
        }

        if (access(file_full_path, R_OK))
        {
            continue;
        }

		md5sum(file_full_path, compute_md5_buff);

		for (i = 0; i< 16; i++) {
			sprintf(convert_md5_buff+(i*2), "%02X", compute_md5_buff[i]);
		}

		if (strncasecmp(file_md5_value, convert_md5_buff, 16)) {
			fail_count++;
			dbg_time("md5 checking: %s fail\n", file_full_path);
			dbg_time("find %s, should be %s\n", file_md5_value, convert_md5_buff);
		} else {
			dbg_time("md5 checking: %s pass\n", file_full_path);
		}

        if (is_upgrade_fimeware_zip_7z)
        {
            dbg_time("%s delet %s ...\n", __func__, file_full_path);
            unlink(file_full_path);   //delete all file
        }
	}

	fclose(fp);
    if (is_upgrade_fimeware_zip_7z)
    {
        unlink(md5_file_path);   //delete md5.txt
    }

	dbg_time("Totals checking %d files md5 value, %d file fail!\n", file_count, fail_count);

	return (fail_count? -1 : 0);
}
